/******************************************************************************
* Copyright (C) 2022 Xilinx, Inc.  All rights reserved.
* SPDX-License-Identifier: MIT
******************************************************************************/
#ifndef XILPLMI_CONFIG_H
#define XILPLMI_CONFIG_H

#include "xparameters.h"

/* #undef PLM_PRINT_NO_UART */
/* #undef PLM_PRINT */
#define	PLM_DEBUG
/* #undef PLM_DEBUG_INFO */
/* #undef PLM_DEBUG_DETAILED */
/* #undef PLM_DEBUG_MODE */
#define	PLM_PRINT_PERF
/* #undef PLM_QSPI_EXCLUDE */
/* #undef PLM_SD_EXCLUDE */
/* #undef PLM_OSPI_EXCLUDE */
/* #undef PLM_SEM_EXCLUDE */
/* #undef PLM_SECURE_EXCLUDE */
#define	PLM_USB_EXCLUDE
#define	PLM_NVM_EXCLUDE
#define	PLM_PUF_EXCLUDE
/* #undef PLM_ENABLE_STL */

#if (defined NUMBER_OF_SLRS)
#define	PLM_ENABLE_PLM_TO_PLM_COMM
#endif

#endif /* XILPLMI_CONFIG_H */
